@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/httparse/1.3.1")]
+#![doc(html_root_url = "https://docs.rs/httparse/1.3.3")]
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(test, deny(warnings))]
 #![deny(missing_docs)]
@@ -55,18 +55,26 @@ fn is_token(b: u8) -> bool {
 }
 
 // ASCII codes to accept URI string.
-// i.e. A-Z a-z 0-9 !#$%&'*+-._();:@=,/?[]~
+// i.e. A-Z a-z 0-9 !#$%&'*+-._();:@=,/?[]~^
 // TODO: Make a stricter checking for URI string?
 static URI_MAP: [bool; 256] = byte_map![
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+//  \0                            \n
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+//  commands
     0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+//  \w !  "  #  $  %  &  '  (  )  *  +  ,  -  .  /
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1,
+//  0  1  2  3  4  5  6  7  8  9  :  ;  <  =  >  ?
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1,
-    0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0,
-    // ====== Extended ASCII (aka. obs-text) ======
+//  @  A  B  C  D  E  F  G  H  I  J  K  L  M  N  O
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1,
+//  P  Q  R  S  T  U  V  W  X  Y  Z  [  \  ]  ^  _
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+//  `  a  b  c  d  e  f  g  h  i  j  k  l  m  n  o
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,
+//  p  q  r  s  t  u  v  w  x  y  z  {  |  }  ~  del
+//   ====== Extended ASCII (aka. obs-text) ======
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -381,6 +389,7 @@ impl<'h, 'b> Response<'h, 'b> {
             },
             b'\r' => {
                 expect!(bytes.next() == b'\n' => Err(Error::Status));
+                bytes.slice();
                 self.reason = Some("");
             },
             b'\n' => self.reason = Some(""),
@@ -558,15 +567,6 @@ pub fn parse_headers<'b: 'h, 'h>(src: &'b [u8], mut dst: &'h mut [Header<'b>])
     Ok(Status::Complete((pos, dst)))
 }
 
-/*
-#[cfg(not(any(
-    feature = "httparse_simd",
-    target_arch = "x86",
-    target_arch = "x86_64",
-    target_feature = "sse4.2"
-)))]
-*/
-
 
 #[inline]
 fn parse_headers_iter<'a, 'b>(headers: &mut &mut [Header<'a>], bytes: &'b mut Bytes<'a>)
@@ -665,21 +665,30 @@ fn parse_headers_iter<'a, 'b>(headers: &mut &mut [Header<'a>], bytes: &'b mut By
             }
 
             //found_ctl
-            if b == b'\r' {
+            let value_slice : &[u8] = if b == b'\r' {
                 expect!(bytes.next() == b'\n' => Err(Error::HeaderValue));
                 count += bytes.pos();
                 // having just check that `\r\n` exists, it's safe to skip those 2 bytes
                 unsafe {
-                    header.value = bytes.slice_skip(2);
+                    bytes.slice_skip(2)
                 }
             } else if b == b'\n' {
                 count += bytes.pos();
                 // having just check that `\r\n` exists, it's safe to skip 1 byte
                 unsafe {
-                    header.value = bytes.slice_skip(1);
+                    bytes.slice_skip(1)
                 }
             } else {
                 return Err(Error::HeaderValue);
+            };
+            // trim trailing whitespace in the header
+            if let Some(last_visible) = value_slice.iter().rposition(|b| *b != b' ' && *b != b'\t' ) {
+                // There is at least one non-whitespace character.
+                header.value = &value_slice[0..last_visible+1];
+            } else {
+                // There is no non-whitespace character. This can only happen when value_slice is
+                // empty.
+                header.value = value_slice;
             }
         }
     } // drop iter
@@ -814,6 +823,28 @@ mod tests {
         }
     }
 
+    req! {
+        test_request_simple_with_query_params,
+        b"GET /thing?data=a HTTP/1.1\r\n\r\n",
+        |req| {
+            assert_eq!(req.method.unwrap(), "GET");
+            assert_eq!(req.path.unwrap(), "/thing?data=a");
+            assert_eq!(req.version.unwrap(), 1);
+            assert_eq!(req.headers.len(), 0);
+        }
+    }
+
+    req! {
+        test_request_simple_with_whatwg_query_params,
+        b"GET /thing?data=a^ HTTP/1.1\r\n\r\n",
+        |req| {
+            assert_eq!(req.method.unwrap(), "GET");
+            assert_eq!(req.path.unwrap(), "/thing?data=a^");
+            assert_eq!(req.version.unwrap(), 1);
+            assert_eq!(req.headers.len(), 0);
+        }
+    }
+
     req! {
         test_request_headers,
         b"GET / HTTP/1.1\r\nHost: foo.com\r\nCookie: \r\n\r\n",
@@ -829,6 +860,21 @@ mod tests {
         }
     }
 
+    req! {
+        test_request_headers_optional_whitespace,
+        b"GET / HTTP/1.1\r\nHost: \tfoo.com\t \r\nCookie: \t \r\n\r\n",
+        |req| {
+            assert_eq!(req.method.unwrap(), "GET");
+            assert_eq!(req.path.unwrap(), "/");
+            assert_eq!(req.version.unwrap(), 1);
+            assert_eq!(req.headers.len(), 2);
+            assert_eq!(req.headers[0].name, "Host");
+            assert_eq!(req.headers[0].value, b"foo.com");
+            assert_eq!(req.headers[1].name, "Cookie");
+            assert_eq!(req.headers[1].value, b"");
+        }
+    }
+
     req! {
         // test the scalar parsing
         test_request_header_value_htab_short,
@@ -1005,6 +1051,19 @@ mod tests {
         }
     }
 
+    res! {
+        test_response_reason_missing_no_space_with_headers,
+        b"HTTP/1.1 200\r\nFoo: bar\r\n\r\n",
+        |res| {
+            assert_eq!(res.version.unwrap(), 1);
+            assert_eq!(res.code.unwrap(), 200);
+            assert_eq!(res.reason.unwrap(), "");
+            assert_eq!(res.headers.len(), 1);
+            assert_eq!(res.headers[0].name, "Foo");
+            assert_eq!(res.headers[0].value, b"bar");
+        }
+    }
+
     res! {
         test_response_reason_with_space_and_tab,
         b"HTTP/1.1 101 Switching Protocols\t\r\n\r\n",
